/*
 * Copyright (c) 2014, Yawning Angel <yawning at schwanenlied dot me>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <array>
#include <iostream>
#include <random>

#include "schwanenlied/crypto/rand_ctr_drbg.h"
#include "gtest/gtest.h"

namespace schwanenlied {
namespace crypto {

class RandCtrDrbgTest : public ::testing::Test {
 protected:
  virtual void SetUp() {}
  virtual void TearDown() {}
};

TEST_F(RandCtrDrbgTest, SmokeTest) {
  RandCtrDrbg rng;

  ::std::uniform_int_distribution<int> foo(1, 6);
  ::std::array<int, 6> counts = {{ 0 }};
  for (auto i = 0; i < 1000; i++) {
    auto sample = foo(rng);
    ASSERT_TRUE(sample >= 1 && sample <= 6);
    counts.at(sample - 1)++;
  }
  ::std::cout << "  1000d6 results: ";
  for (size_t i = 0; i < counts.size(); i++) {
    ::std::cout << (i + 1) << ": " << counts.at(i) << " ";
  }
  ::std::cout << ::std::endl;
}

} // namespace crypto
} // namespace schwanenlied
